@@ -41,38 +41,45 @@
 
 import os
 import sys
-
-from typing.io import TextIO
 from datetime import datetime
+from io import StringIO, TextIOWrapper
 from math import log, sqrt
-import shutil
-import tempfile
-from io import StringIO
-from urllib.request import urlopen, Request
+from typing import Any, Callable, Dict, List, Optional, Union
 from urllib.parse import urlparse, urlunparse
+from urllib.request import Request, urlopen
 
-# Avoid 'from numpy import *' since numpy has lots of names defined
-from numpy import array, asarray, float64, ones, zeros, any
 import numpy as np
 
-from .color import Color
-from .colorscheme import (ColorScheme, SymbolColor, monochrome, hydrophobicity, base_pairing,
-                          chemistry, charge)
-from .logomath import Dirichlet
-
-
-from . import seq_io
-from .data import amino_acid_composition
+# Avoid 'from numpy import *' since numpy has lots of names defined
+from numpy import any, array, asarray, float64, ones, zeros
 from scipy.stats import entropy
-from .seq import (Alphabet, unambiguous_dna_alphabet,
-                  unambiguous_rna_alphabet, unambiguous_protein_alphabet, SeqList)
-from .utils import (isfloat, ArgumentError, stdrepr)
 
-from . import __version__
+from . import __version__, seq_io
+from .color import Color
+from .colorscheme import (
+    ColorScheme,
+    SymbolColor,
+    base_pairing,
+    charge,
+    chemistry,
+    hydrophobicity,
+    monochrome,
+)
+from .data import amino_acid_composition
+from .logomath import Dirichlet
+from .seq import (
+    Alphabet,
+    SeqList,
+    unambiguous_dna_alphabet,
+    unambiguous_protein_alphabet,
+    unambiguous_rna_alphabet,
+)
+from .utils import ArgumentError, isfloat, stdrepr
 
 # Shorten development version string of the form weblogo-3.6.1.dev43+g64d9f12.d20190304
-if __version__.find('+') != -1:
-    __version__ = __version__[:__version__.find('+')]
+
+if __version__.find("+") != -1:
+    __version__ = __version__[: __version__.find("+")]
 
 
 # from .logo_formatter import (GhostscriptAPI, pdf_formatter, jpeg_formatter, png_formatter,
@@ -117,8 +124,9 @@ description = "Create sequence logos from biological sequence alignments."
 release_description = "WebLogo %s" % (__version__)
 
 
-def cgi(htdocs_directory):  # pragma: no cover
+def cgi(htdocs_directory: str) -> None:  # pragma: no cover
     import weblogo._cgi
+
     weblogo._cgi.main(htdocs_directory)
 
 
@@ -126,35 +134,38 @@ aa_composition = [amino_acid_composition[_k] for _k in unambiguous_protein_alpha
 
 # ------  DATA ------
 
-classic = ColorScheme([
-    SymbolColor("G", "orange"),
-    SymbolColor("TU", "red"),
-    SymbolColor("C", "blue"),
-    SymbolColor("A", "green")
-])
+classic = ColorScheme(
+    [
+        SymbolColor("G", "orange"),
+        SymbolColor("TU", "red"),
+        SymbolColor("C", "blue"),
+        SymbolColor("A", "green"),
+    ]
+)
 
-std_color_schemes = {"auto": None,  # Depends on sequence type
-                     "monochrome": monochrome,
-                     "base pairing": base_pairing,
-                     "classic": classic,
-                     "hydrophobicity": hydrophobicity,
-                     "chemistry": chemistry,
-                     "charge": charge,
-                     }
+std_color_schemes = {
+    "auto": None,  # Depends on sequence type
+    "monochrome": monochrome,
+    "base pairing": base_pairing,
+    "classic": classic,
+    "hydrophobicity": hydrophobicity,
+    "chemistry": chemistry,
+    "charge": charge,
+}
 
 default_color_schemes = {
     unambiguous_protein_alphabet: hydrophobicity,
     unambiguous_rna_alphabet: base_pairing,
-    unambiguous_dna_alphabet: base_pairing
+    unambiguous_dna_alphabet: base_pairing,
 }
 
 std_units = {
-    "bits": 1. / log(2),
-    "nats": 1.,
-    "digits": 1. / log(10),
-    "kT": 1.,
-    "kJ/mol": 8.314472 * 298.15 / 1000.,
-    "kcal/mol": 1.987 * 298.15 / 1000.,
+    "bits": 1.0 / log(2),
+    "nats": 1.0,
+    "digits": 1.0 / log(10),
+    "kT": 1.0,
+    "kJ/mol": 8.314472 * 298.15 / 1000.0,
+    "kcal/mol": 1.987 * 298.15 / 1000.0,
     "probability": None,
 }
 
@@ -162,25 +173,22 @@ std_units = {
 # (Courier has a width equal to 3/5 of the point size.)
 # Check that can get 80 characters in journal page @small
 # 40 characters in a journal column
-std_sizes = {
-    "small": 5.4,
-    "medium": 5.4 * 2,
-    "large": 5.4 * 3
-}
+std_sizes = {"small": 5.4, "medium": 5.4 * 2, "large": 5.4 * 3}
 
 std_alphabets = {
-    'protein': unambiguous_protein_alphabet,
-    'rna': unambiguous_rna_alphabet,
-    'dna': unambiguous_dna_alphabet}
+    "protein": unambiguous_protein_alphabet,
+    "rna": unambiguous_rna_alphabet,
+    "dna": unambiguous_dna_alphabet,
+}
 
 std_percentCG = {
-    'H. sapiens': 40.,
-    'E. coli': 50.5,
-    'S. cerevisiae': 38.,
-    'C. elegans': 36.,
-    'D. melanogaster': 43.,
-    'M. musculus': 42.,
-    'T. thermophilus': 69.4,
+    "H. sapiens": 40.0,
+    "E. coli": 50.5,
+    "S. cerevisiae": 38.0,
+    "C. elegans": 36.0,
+    "D. melanogaster": 43.0,
+    "M. musculus": 42.0,
+    "T. thermophilus": 69.4,
 }
 
 
@@ -192,8 +200,8 @@ std_percentCG = {
 # Nat Biotechnol 2004, 22:547-53
 
 
-class LogoOptions(object):
-    """ A container for all logo formatting options. Not all of these
+class LogoOptions:
+    """A container for all logo formatting options. Not all of these
     are directly accessible through the CLI or web interfaces.
 
     To display LogoOption defaults::
@@ -272,8 +280,8 @@ class LogoOptions(object):
 
     """
 
-    def __init__(self, **kwargs):
-        """ Create a new LogoOptions instance.
+    def __init__(self, **kwargs: dict) -> None:
+        """Create a new LogoOptions instance.
 
         >>> logooptions = LogoOptions(logo_title = "Some Title String")
         >>> logooptions.show_yaxis = False
@@ -281,7 +289,7 @@ class LogoOptions(object):
 
         """
 
-        self.alphabet = None
+        self.alphabet: Optional[Alphabet] = None
 
         self.creator_text = release_description
 
@@ -293,10 +301,10 @@ class LogoOptions(object):
 
         self.show_yaxis = True
         # yaxis_lable default depends on other settings. See LogoFormat
-        self.yaxis_label = None
-        self.yaxis_tic_interval = 1.
+        self.yaxis_label: Optional[str] = None
+        self.yaxis_tic_interval = 1.0
         self.yaxis_minor_tic_ratio = 5
-        self.yaxis_scale = None
+        self.yaxis_scale: Optional[float] = None
 
         self.show_xaxis = True
         self.xaxis_label = ""
@@ -317,10 +325,10 @@ class LogoOptions(object):
         self.errorbar_width_fraction = 0.25
         self.errorbar_gray = 0.75
 
-        self.resolution = 96.  # Dots per inch
+        self.resolution: int = 96  # Dots per inch
 
         self.default_color = Color.by_name("black")
-        self.color_scheme = None
+        self.color_scheme: Optional[ColorScheme] = None
         # self.show_color_key = False # NOT yet implemented
 
         self.debug = False
@@ -345,8 +353,8 @@ class LogoOptions(object):
         self.title_font = "ArialMT"
 
         self.first_index = 1
-        self.logo_start = None
-        self.logo_end = None
+        self.logo_start: Optional[int] = None
+        self.logo_end: Optional[int] = None
         self.scale_width = True
 
         self.reverse_stacks = True  # If true, draw stacks with largest letters on top.
@@ -354,7 +362,7 @@ class LogoOptions(object):
         for k, v in kwargs.items():
             setattr(self, k, v)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         attributes = list(vars(self).keys())
         attributes.sort()
         return stdrepr(self, attributes)
@@ -364,11 +372,12 @@ class LogoOptions(object):
 
 
 class LogoFormat(LogoOptions):
-    """ Specifies the format of the logo. Requires LogoData and LogoOptions
+    """Specifies the format of the logo. Requires LogoData and LogoOptions
     objects.
 
     >>> logodata = LogoData.from_seqs(seqs)
     >>> logooptions = LogoOptions()
+
     >>> logooptions.title = "A Logo Title"
     >>> format = LogoFormat(logodata, logooptions)
 
@@ -376,16 +385,14 @@ class LogoFormat(LogoOptions):
         ArgumentError: if arguments are invalid.
     """
 
-    def __init__(self, logodata, logooptions=None):
-        """ Create a new LogoFormat instance.
-
-        """
+    def __init__(self, logodata: "LogoData", logooptions: LogoOptions = None) -> None:
+        """Create a new LogoFormat instance."""
 
         if logooptions is not None:
             self.__dict__.update(logooptions.__dict__)
 
-        self.alphabet = logodata.alphabet
-        self.seqlen = logodata.length
+        self.alphabet: Optional[Alphabet] = logodata.alphabet
+        self.seqlen: Optional[int] = logodata.length
 
         # Derived parameters.
         self.show_title = False
@@ -401,8 +408,8 @@ class LogoFormat(LogoOptions):
         self.xaxis_label_height = None
         self.line_height = None
         self.line_width = None
-        self.logo_height = None
-        self.logo_width = None
+        self.logo_height: Optional[int] = None
+        self.logo_width: Optional[int] = None
         self.creation_date = None
         self.end_type = None
 
@@ -411,21 +418,45 @@ class LogoFormat(LogoOptions):
         # Attribute to test, test, error message
         arg_conditions = (
             ("stacks_per_line", lambda x: x > 0, "Stacks per line must be positive."),
-            ("stack_width", lambda x: x > 0.0, "Stack width must be greater than zero."),
-            ("stack_aspect_ratio", lambda x: x > 0,
-                "Stack aspect ratio must be greater than zero."),
+            (
+                "stack_width",
+                lambda x: x > 0.0,
+                "Stack width must be greater than zero.",
+            ),
+            (
+                "stack_aspect_ratio",
+                lambda x: x > 0,
+                "Stack aspect ratio must be greater than zero.",
+            ),
             ("fontsize", lambda x: x > 0, "Font sizes must be positive."),
             ("small_fontsize", lambda x: x > 0, "Font sizes must be positive."),
             ("title_fontsize", lambda x: x > 0, "Font sizes must be positive."),
-            ("errorbar_fraction", lambda x: x >= 0.0 and x <= 1.0,
-             "The visible fraction of the error bar must be between zero and one."),
-            ("yaxis_tic_interval", lambda x: x >= 0.0,
-                "The yaxis tic interval cannot be negative."),
-            ("yaxis_minor_tic_interval", lambda x: not (x and x < 0.0),
-                "Distances cannot be negative."),
-            ("xaxis_tic_interval", lambda x: x > 0.0, "Tic interval must be greater than zero."),
+            (
+                "errorbar_fraction",
+                lambda x: x >= 0.0 and x <= 1.0,
+                "The visible fraction of the error bar must be between zero and one.",
+            ),
+            (
+                "yaxis_tic_interval",
+                lambda x: x >= 0.0,
+                "The yaxis tic interval cannot be negative.",
+            ),
+            (
+                "yaxis_minor_tic_interval",
+                lambda x: not (x and x < 0.0),
+                "Distances cannot be negative.",
+            ),
+            (
+                "xaxis_tic_interval",
+                lambda x: x > 0.0,
+                "Tic interval must be greater than zero.",
+            ),
             ("number_interval", lambda x: x > 0.0, "Invalid interval between numbers."),
-            ("shrink_fraction", lambda x: x >= 0.0 and x <= 1.0, "Invalid shrink fraction."),
+            (
+                "shrink_fraction",
+                lambda x: x >= 0.0 and x <= 1.0,
+                "Invalid shrink fraction.",
+            ),
             ("stack_margin", lambda x: x > 0.0, "Invalid stack margin."),
             ("logo_margin", lambda x: x > 0.0, "Invalid logo margin."),
             ("stroke_width", lambda x: x > 0.0, "Invalid stroke width."),
@@ -441,25 +472,35 @@ class LogoFormat(LogoOptions):
 
         # Inclusive upper and lower bounds
         # FIXME: Validate here. Move from eps_formatter
+
+        # asserts checks that defaults that were initialized to None have been set
+        assert self.first_index is not None
+        assert self.seqlen is not None
+
         if self.logo_start is None:
             self.logo_start = self.first_index
 
+        assert self.logo_start is not None
+
         if self.logo_end is None:
             self.logo_end = self.seqlen + self.first_index - 1
 
+        assert self.logo_end is not None
+        assert self.logo_start is not None
         self.total_stacks = self.logo_end - self.logo_start + 1
+
         if self.total_stacks <= 0:
-            raise ArgumentError("Logo must contain at least one stack", 'logo_end')
+            raise ArgumentError("Logo must contain at least one stack", "logo_end")
 
         if self.logo_start - self.first_index < 0:
             raise ArgumentError(
-                    "Logo range extends before start of available sequence.",
-                    'logo_range')
+                "Logo range extends before start of available sequence.", "logo_range"
+            )
 
         if self.logo_end - self.first_index >= self.seqlen:
             raise ArgumentError(
-                    "Logo range extends beyond end of available sequence.",
-                    'logo_range')
+                "Logo range extends beyond end of available sequence.", "logo_range"
+            )
 
         if self.logo_title:
             self.show_title = True
@@ -471,6 +512,7 @@ class LogoFormat(LogoOptions):
         if self.yaxis_label is None:
             self.yaxis_label = self.unit_name
 
+        assert self.yaxis_label is not None
         if self.yaxis_label:
             self.show_yaxis_label = True
         else:
@@ -481,20 +523,25 @@ class LogoFormat(LogoOptions):
             conversion_factor = std_units[self.unit_name]
             if conversion_factor:
                 if self.alphabet is None:
-                    raise ArgumentError("Need an alphabet", 'alphabet')
+                    raise ArgumentError("Need an alphabet", "alphabet")
 
+                assert self.alphabet is not None
                 self.yaxis_scale = log(len(self.alphabet)) * conversion_factor
             else:
                 self.yaxis_scale = 1.0  # probability units
 
         if self.yaxis_scale <= 0.0:
-            raise ArgumentError("Invalid yaxis scale", 'yaxis_scale', )
+            raise ArgumentError(
+                "Invalid yaxis scale",
+                "yaxis_scale",
+            )
 
         if self.yaxis_tic_interval >= self.yaxis_scale:
-            self.yaxis_tic_interval /= 2.
+            self.yaxis_tic_interval /= 2.0
 
-        self.yaxis_minor_tic_interval \
-            = float(self.yaxis_tic_interval) / self.yaxis_minor_tic_ratio
+        self.yaxis_minor_tic_interval = (
+            float(self.yaxis_tic_interval) / self.yaxis_minor_tic_ratio
+        )
 
         if self.color_scheme is None:
             if self.alphabet in default_color_schemes:
@@ -537,36 +584,45 @@ class LogoFormat(LogoOptions):
         else:
             self.title_height = 0
 
-        self.xaxis_label_height = 0.
+        self.xaxis_label_height = 0.0
         if self.show_xaxis_label:
             self.xaxis_label_height += self.fontsize
         if self.show_fineprint:
             if len(self.fineprint) != 0:
                 self.xaxis_label_height += self.small_fontsize
 
-        self.line_height = (self.stack_height + self.line_margin_top +
-                            self.line_margin_bottom)
-        self.line_width = (self.stack_width * self.stacks_per_line +
-                           self.line_margin_left + self.line_margin_right)
+        self.line_height = (
+            self.stack_height + self.line_margin_top + self.line_margin_bottom
+        )
+        self.line_width = (
+            self.stack_width * self.stacks_per_line
+            + self.line_margin_left
+            + self.line_margin_right
+        )
 
-        self.logo_height = int(2 * self.logo_margin + self.title_height
-                               + self.xaxis_label_height + self.line_height * self.lines_per_logo)
+        self.logo_height = int(
+            2 * self.logo_margin
+            + self.title_height
+            + self.xaxis_label_height
+            + self.line_height * self.lines_per_logo
+        )
         self.logo_width = int(2 * self.logo_margin + self.line_width)
 
-        self.creation_date = datetime.now().isoformat(' ')
+        self.creation_date = datetime.now().isoformat(" ")
 
-        end_type = '-'
+        end_type = "-"
         end_types = {
-            unambiguous_protein_alphabet: 'p',
-            unambiguous_rna_alphabet: '-',
-            unambiguous_dna_alphabet: 'd'
+            unambiguous_protein_alphabet: "p",
+            unambiguous_rna_alphabet: "-",
+            unambiguous_dna_alphabet: "d",
         }
         if self.show_ends and self.alphabet in end_types:
             end_type = end_types[self.alphabet]
         self.end_type = end_type
 
         if self.annotate is None:
-            self.annotate = []
+            self.annotate: list = []
+
             for i in range(self.seqlen):
                 index = i + self.first_index
                 if index % self.number_interval == 0:
@@ -576,8 +632,8 @@ class LogoFormat(LogoOptions):
 
         if len(self.annotate) != self.seqlen:
             raise ArgumentError(
-                    "Annotations must be same length as sequences.",
-                    'annotate')
+                "Annotations must be same length as sequences.", "annotate"
+            )
 
     # End __init__
 
@@ -585,13 +641,15 @@ class LogoFormat(LogoOptions):
 # End class LogoFormat
 
 
-def parse_prior(composition, alphabet, weight=None):
-    """ Parse a description of the expected monomer distribution of a sequence.
+def parse_prior(
+    composition: Any, alphabet: Alphabet, weight: float = None
+) -> Optional[np.ndarray]:
+    """Parse a description of the expected monomer distribution of a sequence.
 
     Valid compositions:
 
     * None or 'none'
-        No composition sepecified
+        No composition specified
     * 'auto' or 'automatic'
         Use the typical average distribution
         for proteins and an equiprobable distribution for
@@ -604,12 +662,15 @@ def parse_prior(composition, alphabet, weight=None):
         Use the average CG percentage for the species's genome.
     * An explicit distribution
         e.g. {'A':10, 'C':40, 'G':40, 'T':10}
+
+    returns a dict of {monomer: probability} pairs.
     """
     if composition is None:
         return None
+
     comp = composition.strip()
 
-    if comp.lower() == 'none':
+    if comp.lower() == "none":
         return None
 
     if weight is None and alphabet is not None:
@@ -618,9 +679,11 @@ def parse_prior(composition, alphabet, weight=None):
     if weight < 0:
         raise ValueError("Weight cannot be negative.")
 
-    if comp.lower() == 'equiprobable':
+    if comp.lower() == "equiprobable":
+        prior: np.ndarray
         prior = weight * equiprobable_distribution(len(alphabet))
-    elif comp.lower() == 'auto' or comp.lower() == 'automatic':
+
+    elif comp.lower() == "auto" or comp.lower() == "automatic":
         if alphabet == unambiguous_protein_alphabet:
             prior = weight * asarray(aa_composition, float64)
         else:
@@ -629,20 +692,25 @@ def parse_prior(composition, alphabet, weight=None):
     elif comp in std_percentCG:
         prior = weight * base_distribution(std_percentCG[comp])
 
-    elif comp[-1] == '%':
+    elif comp[-1] == "%":
         prior = weight * base_distribution(float(comp[:-1]))
 
     elif isfloat(comp):
-        prior = weight * base_distribution(float(comp) * 100.)
+        prior = weight * base_distribution(float(comp) * 100.0)
 
-    elif composition[0] == '{' and composition[-1] == '}':
-        explicit = composition[1: -1]
-        explicit = explicit.replace(',', ' ').replace("'", ' ').replace('"', ' ') \
-            .replace(':', ' ').split()
+    elif composition[0] == "{" and composition[-1] == "}":
+        explicit = composition[1:-1]
+        explicit = (
+            explicit.replace(",", " ")
+            .replace("'", " ")
+            .replace('"', " ")
+            .replace(":", " ")
+            .split()
+        )
 
         if len(explicit) != len(alphabet) * 2:
             raise ValueError("Explicit prior does not match length of alphabet")
-        prior = - ones(len(alphabet), float64)
+        prior = -ones(len(alphabet), float64)
         try:
             for r in range(len(explicit) // 2):
                 letter = explicit[r * 2]
@@ -652,8 +720,10 @@ def parse_prior(composition, alphabet, weight=None):
         except ValueError:
             raise ValueError("Cannot parse explicit composition")
 
-        if any(prior == -1.):
-            raise ValueError("Explicit prior does not match alphabet")  # pragma: no cover
+        if any(prior == -1.0):
+            raise ValueError(
+                "Explicit prior does not match alphabet"
+            )  # pragma: no cover
         prior /= sum(prior)
         prior *= weight
 
@@ -662,15 +732,16 @@ def parse_prior(composition, alphabet, weight=None):
 
     if len(prior) != len(alphabet):
         raise ValueError(
-                "The sequence alphabet and composition are incompatible.")  # pragma: no cover
+            "The sequence alphabet and composition are incompatible."
+        )  # pragma: no cover
     return prior
 
 
-def base_distribution(percentCG):
-    A = (1. - (percentCG / 100.)) / 2.
-    C = (percentCG / 100.) / 2.
-    G = (percentCG / 100.) / 2.
-    T = (1. - (percentCG / 100)) / 2.
+def base_distribution(percentCG: float) -> np.ndarray:
+    A = (1.0 - (percentCG / 100.0)) / 2.0
+    C = (percentCG / 100.0) / 2.0
+    G = (percentCG / 100.0) / 2.0
+    T = (1.0 - (percentCG / 100)) / 2.0
     return asarray((A, C, G, T), float64)
 
 
@@ -678,30 +749,32 @@ def equiprobable_distribution(length: int) -> np.ndarray:
     return ones((length), float64) / length
 
 
-def _seq_formats():
-    """ Return a dictionary mapping between the names of formats for the sequence data
-    and the corresponing parsers.
+def _seq_formats() -> Dict[str, str]:
+    """Return a dictionary mapping between the names of formats for the sequence data
+    and the corresponding parsers.
     """
     # Add position weight matrix formats to input parsers by hand
     fin_choices = dict(seq_io.format_names())
-    fin_choices['transfac'] = 'transfac'
-    del fin_choices['plain']
+    fin_choices["transfac"] = "transfac"
+    del fin_choices["plain"]
     return fin_choices
 
 
-def _seq_names():
-    """ Returns a list of the names of accepted sequence data formats."""
-    fin_names = [f.names[0] for f in seq_io.formats]
-    fin_names.remove('plain')
-    fin_names.append('transfac')
+def _seq_names() -> List[str]:
+    """Returns a list of the names of accepted sequence data formats."""
+    fin_names = [f.names[0] for f in seq_io.formats]  # type: ignore
+    fin_names.remove("plain")
+    fin_names.append("transfac")
     return fin_names
 
 
-def read_seq_data(fin,
-                  input_parser=seq_io.read,
-                  alphabet=None,
-                  ignore_lower_case=False,
-                  max_file_size=0):
+def read_seq_data(
+    fin: Union[StringIO, TextIOWrapper, None],
+    input_parser: Callable = seq_io.read,
+    alphabet: Alphabet = None,
+    ignore_lower_case: bool = False,
+    max_file_size: int = 0,
+) -> SeqList:
     """Read sequence data from the input stream and return a seqs object.
 
     The environment variable WEBLOGO_MAX_FILE_SIZE overides the max_file_size argument.
@@ -712,8 +785,11 @@ def read_seq_data(fin,
 
     # If max_file_size is set, or if fin==stdin (which is non-seekable), we
     # read the data and replace fin with a StringIO object.
-    if (max_file_size > 0):
+    assert fin is not None
+
+    if max_file_size > 0:
         data = fin.read(max_file_size)
+
         more_data = fin.read(2)
         if more_data != "":
             raise IOError("File exceeds maximum allowed size: %d bytes" % max_file_size)
@@ -740,7 +816,7 @@ def read_seq_data(fin,
     return seqs
 
 
-class LogoData(object):
+class LogoData:
     """The data needed to generate a sequence logo.
 
     Args:
@@ -750,10 +826,17 @@ class LogoData(object):
         counts:   An array of character counts
         entropy:  The relative entropy of each column
         entropy_interval: entropy confidence interval
-     """
+    """
 
-    def __init__(self, length=None, alphabet=None, counts=None,
-                 entropy=None, entropy_interval=None, weight=None):
+    def __init__(
+        self,
+        length: Optional[int] = None,
+        alphabet: Optional[Alphabet] = None,
+        counts: np.ndarray = None,
+        entropy: np.ndarray = None,
+        entropy_interval: np.ndarray = None,
+        weight: Optional[np.ndarray] = None,
+    ) -> None:
         """Creates a new LogoData object"""
         self.length = length
         self.alphabet = alphabet
@@ -763,10 +846,9 @@ class LogoData(object):
         self.weight = weight
 
     @classmethod
-    def from_counts(cls,
-                    alphabet: Alphabet,
-                    counts: np.ndarray,
-                    prior: np.ndarray = None) -> 'LogoData':
+    def from_counts(
+        cls, alphabet: Optional[Alphabet], counts: np.ndarray, prior: np.ndarray = None
+    ) -> "LogoData":
         """Build a LogoData object from counts."""
         # Counts is a Motif object?
         # counts = counts.array
@@ -799,19 +881,21 @@ class LogoData(object):
 
                 posterior = Dirichlet(alpha)
                 ent[i] = posterior.mean_relative_entropy(prior / sum(prior))
-                entropy_interval[i][0], entropy_interval[i][1] = \
-                    posterior.interval_relative_entropy(prior / sum(prior), 0.95)
+                (
+                    entropy_interval[i][0],
+                    entropy_interval[i][1],
+                ) = posterior.interval_relative_entropy(prior / sum(prior), 0.95)
 
         weight = array(np.sum(counts, axis=1), float)
         max_weight = max(weight)
         if max_weight == 0.0:
-            raise ValueError('No counts.')
+            raise ValueError("No counts.")
         weight /= max_weight
 
         return cls(seq_length, alphabet, counts, ent, entropy_interval, weight)
 
     @classmethod
-    def from_seqs(cls, seqs: SeqList, prior: np.ndarray = None) -> 'LogoData':
+    def from_seqs(cls, seqs: SeqList, prior: np.ndarray = None) -> "LogoData":
         """Build a LogoData object from a SeqList, a list of sequences."""
         # --- VALIDATE DATA ---
         # check that at least one sequence of length at least 1 long
@@ -825,8 +909,10 @@ class LogoData(object):
             # TODO: Redundant? Should be checked in SeqList?
             if seq_length != len(s):
                 raise ArgumentError(
-                        "Sequence number %d differs in length from the previous sequences"
-                        % (i + 1), 'sequences')
+                    "Sequence number %d differs in length from the previous sequences"
+                    % (i + 1),
+                    "sequences",
+                )
 
         # FIXME: Check seqs.alphabet?
 
@@ -835,41 +921,48 @@ class LogoData(object):
 
     def __str__(self) -> str:
         out = StringIO()
-        print('## LogoData', file=out)
-        print('# First column is position number, counting from zero', file=out)
-        print('# Subsequent columns are raw symbol counts', file=out)
-        print('# Entropy is mean entropy measured in nats.', file=out)
-        print('# Low and High are the 95% confidence limits.', file=out)
-        print('# Weight is the fraction of non-gap symbols in the column.', file=out)
-        print('#\t', file=out)
+        print("## LogoData", file=out)
+        print("# First column is position number, counting from zero", file=out)
+        print("# Subsequent columns are raw symbol counts", file=out)
+        print("# Entropy is mean entropy measured in nats.", file=out)
+        print("# Low and High are the 95% confidence limits.", file=out)
+        print("# Weight is the fraction of non-gap symbols in the column.", file=out)
+        print("#\t", file=out)
         # Show column names
-        print('#', end='\t', file=out)
+        print("#", end="\t", file=out)
+
+        # asserts checks that defaults that were initialized to None have been set
+        assert self.alphabet is not None
+        assert self.length is not None
+        assert self.counts is not None
+        assert self.entropy is not None
+
         for a in self.alphabet:
-            print(a, end=' \t', file=out)
-        print('Entropy\tLow\tHigh\tWeight', file=out)
+            print(a, end=" \t", file=out)
+        print("Entropy\tLow\tHigh\tWeight", file=out)
 
         # Write the data table
+
         for i in range(self.length):
-            print(i + 1, end=' \t', file=out)
+            print(i + 1, end=" \t", file=out)
+
             for c in self.counts[i]:
-                print(c, end=' \t', file=out)
-            print("%6.4f" % self.entropy[i], end=' \t', file=out)
+                print(c, end=" \t", file=out)
+            print("%6.4f" % self.entropy[i], end=" \t", file=out)
             if self.entropy_interval is not None:
-                print("%6.4f" % self.entropy_interval[i][0], end=' \t',
-                      file=out)
-                print("%6.4f" % self.entropy_interval[i][1], end=' \t',
-                      file=out)
+                print("%6.4f" % self.entropy_interval[i][0], end=" \t", file=out)
+                print("%6.4f" % self.entropy_interval[i][1], end=" \t", file=out)
             else:
-                print('\t', '\t', end='', file=out)
+                print("\t", "\t", end="", file=out)
             if self.weight is not None:
-                print("%6.4f" % self.weight[i], end='', file=out)
-            print('', file=out)
-        print('# End LogoData', file=out)
+                print("%6.4f" % self.weight[i], end="", file=out)
+            print("", file=out)
+        print("# End LogoData", file=out)
 
         return out.getvalue()
 
 
-def _from_URL_fileopen(target_url: str) -> TextIO:  # pragma: no cover
+def _from_URL_fileopen(target_url: str) -> StringIO:  # pragma: no cover
     """opens files from a remote URL location"""
 
     # parsing url in component parts
@@ -880,7 +973,7 @@ def _from_URL_fileopen(target_url: str) -> TextIO:  # pragma: no cover
         raise ValueError("Cannot open url: %s", target_url)
 
     # checks for dropbox link
-    if net_location == 'www.dropbox.com':
+    if net_location == "www.dropbox.com":
         # changes dropbox http link into download link
         if query == "dl=0":
             query2 = "dl=1"
@@ -889,13 +982,13 @@ def _from_URL_fileopen(target_url: str) -> TextIO:  # pragma: no cover
         target_url = urlunparse((scheme, net_location, path, param, query2, ""))
 
     # checks for google drive link
-    if net_location == 'drive.google.com':
+    if net_location == "drive.google.com":
         # link configuration for direct download instead of html frame
         google_directdl_frag = "https://docs.google.com/uc?export=download&id="
 
         # pull file id
         (scheme, net_location, path_raw, param, query, frag) = urlparse(target_url)
-        id_file = path_raw.split('/')[3]
+        id_file = path_raw.split("/")[3]
 
         # rebuild URL for direct download
         target_url = google_directdl_frag + id_file
@@ -903,7 +996,5 @@ def _from_URL_fileopen(target_url: str) -> TextIO:  # pragma: no cover
     # save url to temporary file
     req = Request(target_url)
     res = urlopen(req)
-    temp = tempfile.TemporaryFile()
-    shutil.copyfileobj(res, temp)
-    temp.seek(0)
-    return temp
+
+    return StringIO(str(res.read()))
